[{"locale":"es"},{"key":"W","names":["W","w"],"mappings":{"default":{"default":"watt"}},"category":"energy"},{"key":"kW","names":["kw","kW"],"mappings":{"default":{"default":"kilowatt"}},"category":"energy"},{"key":"mW","names":["mw","mW"],"mappings":{"default":{"default":"miliwatt"}},"category":"energy"},{"key":"kwh","names":["kwh","kWh"],"mappings":{"default":{"default":"kilowatt hora"}},"category":"energy"},{"key":"J","names":["J"],"mappings":{"default":{"default":"Joules"}},"category":"energy"},{"key":"N","names":["N"],"mappings":{"default":{"default":"Newton"}},"category":"energy"},{"key":"A","names":["A"],"mappings":{"default":{"default":"Ampere"}},"category":"energy"},{"key":"V","names":["V"],"mappings":{"default":{"default":"Voltio"}},"category":"energy"},{"key":"µΩ","names":["µΩ"],"mappings":{"default":{"default":"microohmio"}},"category":"energy"},{"key":"mΩ","names":["mΩ"],"mappings":{"default":{"default":"miliohmio"}},"category":"energy"},{"key":"Ω","names":["Ω","Ohm"],"mappings":{"default":{"default":"ohmio"}},"category":"energy"},{"key":"kΩ","names":["kΩ","KΩ"],"mappings":{"default":{"default":"kiloohmio"}},"category":"energy"},{"key":"MΩ","names":["MΩ"],"mappings":{"default":{"default":"megaohmio"}},"category":"energy"},{"key":"GΩ","names":["GΩ"],"mappings":{"default":{"default":"gigaohmio"}},"category":"energy"}]
